package org.apithefire.util.assertion;

import java.util.ArrayList;
import java.util.List;

public class ExceptionAssertion {
	
	private interface Checker {
		public void check(Throwable exception);
	}

	private final List<Checker> checkers = new ArrayList<Checker>();
	
	private void addChecker(Checker checker) {
		if (checker == null) {
			throw new NullPointerException();
		}
		checkers.add(checker);
	}
	
	protected ExceptionAssertion() {
	}
	
	public ExceptionAssertion type(final Class<? extends Throwable> type) {
		if (type == null) {
			throw new NullPointerException();
		}
		addChecker(new Checker() {
			public void check(Throwable exception) {
				if (!type.isAssignableFrom(exception.getClass())) {
					throw new AssertionError(
							"Expected exception " + type.getName()
							+ " not thrown but was "
							+ exception.getClass().getName() + ".");
				}
			}
		});
		return this;
	}
	
	public ExceptionAssertion messageContains(final String string) {
		if (string == null) {
			throw new NullPointerException();
		}
		addChecker(new Checker() {
			public void check(Throwable exception) {
				String message = exception.getMessage();
				
				if (message == null
						|| !message.toUpperCase().contains(string.toUpperCase())) {
					throw new AssertionError(
							"Exception message does not contains: " + string);
				}
			}
		});
		return this;
	}
	
	public void in(Runnable runnable) {
		Throwable caught = null;
		try {
			runnable.run();
		} catch (Throwable e) {
			caught = e;
		}
		if (caught == null) {
			throw new AssertionError("No exception thrown.");
		}
		for (Checker checker : checkers) {
			checker.check(caught);
		}
	}
	
}
