package org.apithefire.util.lang;

public class ToStringBuilder {

	private final Object object;
	private final StringBuilder properties;
	
	protected ToStringBuilder(Object object) {
		this.object = Objects.nonNull(object);
		this.properties = new StringBuilder();
	}
	
	private ToStringBuilder propertyRaw(String name, String value) {
		if (properties.length() > 0) {
			properties.append(",");
		}
		properties.append(name);
		properties.append('=');
		properties.append(value);
		return this;
	}
	
	public ToStringBuilder property(String name, boolean value) {
		return propertyRaw(Objects.nonNull(name), Boolean.toString(value));
	}
	
	public ToStringBuilder property(String name, char value) {
		return propertyRaw(Objects.nonNull(name), Character.toString(value));
	}
	
	public ToStringBuilder property(String name, short value) {
		return propertyRaw(Objects.nonNull(name), Short.toString(value));
	}
	
	public ToStringBuilder property(String name, int value) {
		return propertyRaw(Objects.nonNull(name), Integer.toString(value));
	}
	
	public ToStringBuilder property(String name, long value) {
		return propertyRaw(Objects.nonNull(name), Long.toString(value));
	}
	
	public ToStringBuilder property(String name, float value) {
		return propertyRaw(Objects.nonNull(name), Float.toString(value));
	}
	
	public ToStringBuilder property(String name, double value) {
		return propertyRaw(Objects.nonNull(name), Double.toString(value));
	}
	
	public ToStringBuilder property(String name, Object value) {
		if (value == null) {
			return propertyRaw(Objects.nonNull(name), null);
		}
		if (value instanceof String) {
			return propertyRaw(Objects.nonNull(name), (String) value);
		}
		return propertyRaw(Objects.nonNull(name), value.toString());
	}
	
	public ToStringBuilder property(String name, String value) {
		return propertyRaw(Objects.nonNull(name),
				Strings.doubleQuoteEscaped(value));
	}
	
	@Override
	public String toString() {
		if (properties.length() == 0) {
			return object.toString();
		}
		
		StringBuilder full = new StringBuilder();
		
		full.append(object.getClass().getSimpleName());
		full.append('[');
		full.append(properties);
		full.append(']');
		return full.toString();
	}
	
}
