package org.apithefire.util.resource;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.JarURLConnection;
import java.net.URI;
import java.net.URL;
import java.util.Date;
import java.util.List;
import java.util.jar.JarFile;

import org.apithefire.util.lang.Path;
import org.apithefire.util.lang.Strings;
import org.apithefire.util.runtimeexception.io.RuntimeIoException;

public class ClassResource extends AbstractResource {
	
	private AbstractResource resource;
	
	public ClassResource(Class<?> clazz) {
		this(clazz.getClassLoader(), clazz.getPackage());
	}
	
	public ClassResource() {
		this(ClassLoader.getSystemClassLoader(), null);
	}
	
	public ClassResource(ClassLoader classLoader) {
		this(classLoader, null);
	}

	protected ClassResource(ClassLoader classLoader, Package pkg) {
		URL baseUrl = classLoader.getResource("");
		String protocol = baseUrl.getProtocol();
		
		if ("file".equals(protocol)) {
			resource = createFileResource(baseUrl);
		} else if ("jar".equals(protocol)) {
			resource = createJarResource(baseUrl);
		} else {
			throw new ResourceException("Unrecognized protocol " + protocol);
		}
		if (pkg != null) {
			String pkgName = Strings.emptyIfNull(pkg.getName());
			Path pkgPath = Path.split(pkgName, ".").trim();
			
			if (!pkgPath.isEmpty()) {
				resource = (AbstractResource) resource.getResource(pkgPath);
			}
		}
	}
	
	private static AbstractResource createFileResource(URL url) {
		File file = new File(URI.create(url.toString()));
		return new FileResource(file);
	}
	
	private static AbstractResource createJarResource(URL url) {
		try {
			JarURLConnection conn = (JarURLConnection) url.openConnection();
			JarFile jarFile = conn.getJarFile();
			JarResource resource = new JarResource(jarFile);
			String entryName = conn.getEntryName();
			
			if (entryName == null) {
				return resource;
			}
			return (AbstractResource) resource.getResource(Path.split(entryName));
		} catch (IOException e) {
			throw new RuntimeIoException(e);
		}
	}

	public boolean isDirectory() {
		return resource.isDirectory();
	}

	public List<String> listResourceNames(NameFilter filter) {
		return resource.listResourceNames(filter);
	}
	
	@Override
	public Resource doGetResource(Path relativePath) {
		return resource.doGetResource(relativePath);
	}

	public boolean isContent() {
		return resource.isContent();
	}

	public Date getContentModifiedTime() {
		return resource.getContentModifiedTime();
	}

	public long getContentSize() {
		return resource.getContentSize();
	}

	public InputStream openContent() {
		return resource.openContent();
	}

}
