package org.apithefire.util.resource;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;
import org.apithefire.util.runtimeexception.io.RuntimeIoException;

class ContentJarResource extends AbstractResource {
	
	private final JarFile jarFile;
	private final JarEntry jarEntry;
	
	public ContentJarResource(JarFile jarFile, JarEntry jarEntry) {
		this.jarFile = Objects.nonNull(jarFile);
		this.jarEntry = Objects.nonNull(jarEntry);
	}
	
	public boolean isDirectory() {
		return false;
	}

	public List<String> listResourceNames(NameFilter filter) {
		Objects.nonNull(filter);
		return Collections.emptyList();
	}

	@Override
	public Resource doGetResource(Path relativePath) {
		throw ResourceExceptions.resourceNotFoundException(relativePath);
	}
	
	public boolean isContent() {
		return true;
	}

	public Date getContentModifiedTime() {
		return new Date(jarEntry.getTime());
	}

	public long getContentSize() {
		long length = jarEntry.getSize();
		
		return length > 0 ? length : 0;
	}

	public InputStream openContent() {
		try {
			return jarFile.getInputStream(jarEntry);
		} catch (IOException e) {
			throw new RuntimeIoException(e);
		}
	}
	
	@Override
	public String toString() {
		return Objects.toString(this)
				.property("jarFile", jarFile)
				.property("path", jarEntry.getName())
				.toString();
	}

}
