package org.apithefire.util.resource;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;
import org.apithefire.util.runtimeexception.io.RuntimeIoException;

public class FileResource extends AbstractResource {
	
	// XXX: Is this required?
	public interface FileFactory {
		public File create(File parent, String name);
	}
	
	private static final FileFactory defaultFileFactory = new FileFactory() {
		public File create(File parent, String name) {
			return new File(parent, name);
		}
	};
	
	private final File file;
	private final FileFactory fileFactory;
	
	public FileResource(File file) {
		this.file = Objects.nonNull(file);
		this.fileFactory = defaultFileFactory;
	}

	protected FileResource(File file, FileFactory fileFactory) {
		this.file = Objects.nonNull(file);
		this.fileFactory = Objects.nonNull(fileFactory);
	}
	
	public File getFile() {
		return file;
	}
	
	public boolean isDirectory() {
		return file.isDirectory();
	}

	public List<String> listResourceNames(NameFilter filter) {
		String[] fileNames = file.list();
		
		if (fileNames == null) {
			return Collections.emptyList();
		}
		
		List<String> resourceNames = new ArrayList<String>(fileNames.length);
		
		for (String fileName : fileNames) {
			String resourceName = filter.filter(fileName);
			
			if (resourceName != null && resourceName.length() > 0) {
				resourceNames.add(resourceName);
			}
		}
		return resourceNames;
	}

	@Override
	protected Resource doGetResource(Path relativePath) {
		File currentFile = file;
		
		// XXX: bad performance
		for (int i = 0; i < relativePath.length(); ++i) {
			String pathComp = relativePath.componentAt(i);
			
			if (pathComp.equals(".") || pathComp.equals("..")) {
				throw ResourceExceptions.resourceNotFoundException(relativePath);
			}
			currentFile = fileFactory.create(currentFile, pathComp);
		}
		if (!currentFile.exists()) {
			throw ResourceExceptions.resourceNotFoundException(relativePath);
		}
		return new FileResource(currentFile, fileFactory);
	}
	
	public boolean isContent() {
		return file.isFile();
	}
	
	public Date getContentModifiedTime() {
		return new Date(file.lastModified());
	}
	
	public long getContentSize() {
		long length = file.length();
		
		return length > 0 ? length : 0;
	}

	public InputStream openContent() {
		try {
			return new FileInputStream(file);
		} catch (FileNotFoundException e) {
			throw new RuntimeIoException(e);
		}
	}
	
	@Override
	public String toString() {
		return Objects.toString(this).property("file", file).toString();
	}

}
